fs = require 'fs'


PostSchema = new app.mongoose.Schema
  title:  String
  topic:  String
  body:   String
  slug:   String
  date:   Date  
  
PostSchema.method 'update', (text) ->
  lines = text.split('\n')
  @title = lines.first()
  @topic = lines[1]
  @date  = lines.last().toDate()
  @body  = lines.slice(2, lines.length - 1).join('\n')
  this.save()
  
  
PostSchema.static 'update_or_create', (file, callback) ->
  slug = file.name.split('.')[0]
  text = fs.readFileSync file.path, 'utf-8'
  
  Post.findOne {slug: slug}, (err, post) ->
    unless post
      post = new Post
      post.slug = slug
    
    post.update(text)
    callback()

PostSchema.static 'topics', (callback) ->
  Post.find {}, (err, posts) ->
    topics = []
    Object.each posts.groupBy('topic'), (topic, list) ->
      topics.push
        name: topic
        posts: list.sortBy('date', desc=true)
      
    topics = topics.sortBy ((t) -> t.posts.first().date), desc=true
    callback(topics)


Post = app.mongoose.model('Post', PostSchema)


app.get '/', (req, res) ->
  Post.topics (topics) ->
    req.topics = topics
    res.render 'index', req


app.get '/posts/:slug', (req, res) ->
  Post.findOne {slug: req.params.slug}, (err, post) ->
    req.title = post.title
    req.post = post
    res.render 'post', req


app.post '/posts', (req, res) ->
  req.form.complete (err, fields, files) ->
    if err
      next(err)
    else
      file = files.file
      Post.update_or_create file, ->
        res.send 'ok\n'


app.delete '/posts/:slug', (req, res) ->
  Post.findOne {slug: req.params.slug}, (err, post) ->
    if post
      post.remove ->
        res.send "ok\n"
    else  
      res.send "not found\n"


module.exports = Post

